#!/usr/bin/perl
=pod

=head1 NAME

gen-ident-exceptions.pl - a script to gather a names of symbols exported
by external modules that a set of perl files uses.

=head1 SYNOPSIS

B<gen-ident-exceptions.pl> 
    S<[ B<--include-symbols-from-exporter> ] >
    S<[ B<-p> I<perl-path> ] [ B<-d> I<dir-prefix-to-exclude> ]..>
    S<[ B<-u> I<user-exceptions-file> ] ..>   
    S<[ B<-U> I<user-exception-comma-sep-list> ] ..>
    S<[ B<-m> I<module-exceptions-file> ] ..> 
    S<[ B<-M> I<module-exception-comma-sep-list> ] ..>
    S<[ B<-n> I<modulenames-to-skip-file> ] ..> 
    S<[ B<-N> I<modulenames-to-skip-comma-sep-list> ] ..>
    S<[ B<-x> I<not-exceptions-file> ] ..>   
    S<[ B<-X> I<not-exception-comma-sep-list> ] ..    >
    S<[ B<-l> I<name-of-file-with-list-of-files> ] ..    >
    S<I<destfile> I<srcfiles>..>

=head1 DESCRIPTION

It is obvious that when obfuscating some perl file, the names of
symbols the from the modules it imports should not be mangled (or obfuscated
versions of all dependant modules needs to be shipped, which is not always
convenient). The only way of excluding the symbol names from being mangled
is to list them in I<file with list of exceptions>. This utility is dedicated
to creation of such I<file with list of exceptions>.

=head1 OPTIONS

The options after which there is a ellipsis (I<..>) in the synopsis section
accept a set of strings and thus may be specified several times. The values 
passed to such options will 
be merged. The values for all those options are strings that do not contain
spaces and commas in them. If the value of such option contains only one
space or comma, 
it is treated as there were two occurencies of that option with values formed 
from strings before and after that space or comma character. I.e. the 
following commandline strings will equally alter the behaviour of this 
script for option named B<-X>:

    -X a -X b -X c -X d
    -X 'a b' -X c -X d
    -X 'a b c d'
    -X 'a,b' -X c -X d
    -X 'a,b,c,d'


=over 4

=item B<-p> I<perl-path>

Specify the invokation string for the backend. It does not need to specify
the full path of the perl program. This option is intended for passing any 
additional options mostly used for specifying the location of modules your
program requires. The passed string is prepended to the necessary options
used for functionality of this script. The default value of this option is
I<perl> and is sufficient for most cases.

=item B<-d> I<dir-prefix-to-exclude>

Specify the set of directory name prefixes modules from which will be the
modules whoose symbols will be added to I<file with list of exceptions>.
If you need to generate a list of a exceptions for a file in the current
directory and you are unix-like OS, then it's very convenient to use the 
prefix I</> to get a list of all exceptions (you will get a list of 
symbols from the modules that are located not in the current directory - e.g
F</usr/lib/perl5/5.6.1/Pod/Usage.pm>).

=item B<-u> I<user-exceptions-file>

=item B<-U> I<user-exception-comma-sep-list>

The B<-u> option specifies filenames from which a list of symbols (each line
of the file lists exactly one symbol name probably surrounded by spaces) 
that should be added to resultant set of exceptions is taken. 
E.g. the symbols
I<blah>, I<foo>, I<bar> will be extracted from the file with 
the following content:

 blah
   foo
  bar

The B<-U> option specifies the a list of symbols 
that should be added to resultant set of exceptions. 
E.g. all the following commandline fragments request to add symbols 
I<blah>, I<foo>, I<bar>, I<baz>:

    -U blah -U foo -U bar -U  baz
    -U 'blah foo' -U 'bar,baz'
    -U 'blah foo bar baz'

=item B<--include-symbols-from-exporter>

If this option is specified, symbols listed as defined by Exporter::Heavy
are not ignored. Previous versions of this utility included such symbols
as exceptions; this commandline switch is added as a mean to active old
behaviour.

=item B<-m> I<module-exceptions-file>

=item B<-M> I<module-exceptions-comma-sep-list>

The B<-m> option specifies filenames from which a list of module name prefixes
symbols from which should be added to resultant set of exceptions is taken.
The B<-M> option specifies the module name prefixes but immediately on 
commandline. 

Module with name I<$name> is considered I<source for exceptions>
if C<(split('::',$name))[0]> was exactly equal (i.e. charcter-by-character, 
case-sensitive - in other words using C<eq> perl operator)
to the ones 
specifed directly via B<-M> or mentioned in file specified using B<-m> option.

For example, specifying C<-M App> will make all symbols from modules
named I<App::Manager>, I<App::Packer> listed as exceptions, but not symbols
from modules I<AppConfig::MyFile> and I<AppConfig::Std>.

=item B<-n> I<modulenames-to-skip-file>

=item B<-N> I<modulenames-to-skip-comma-sep-list>

The B<-n> option specifies filenames from which a list of module name prefixes
symbols from which should be excluded from resultant set of exceptions is taken.
The B<-N> option specifies the module name prefixes treated this way 
but immediately on  commandline. 

Module with name I<$name> is not considered I<source for exceptions>
if C<(split('::',$name))[0]> was exactly equal (i.e. charcter-by-character, 
case-sensitive - in other words using C<eq> perl operator)
to the ones 
specifed directly via B<-N> or mentioned in file specified using B<-n> option.

=item B<-x> I<not-exceptions-file>

=item B<-X> I<not-exception-comma-sep-list>

The B<-x> option specifies names of files from which a names of symbols to B<omit>
from output exceptions list is taken.
The B<-X> option specifies the names of symbols to B<omit>
from output exceptions list immediately on commandline. 

Use these options if you are sure that the code being obfuscated won't ever use
symbols with these names from B<other> modules (using these names for
private variables is perfectly OK, of course) - this helps your programs to 
become even more obfuscated, especially if you use modules that export a lot
of symbols (like e.g. I<Gtk::*>).

=item B<-l> I<name-of-file-with-list-of-files> 

Pass name of file that contains name of source files to process.
The names of files should be listed there one per line.
Use this option if you have too many files so resultant command line gets too long
for your operating system.

=back 

=head1 NOTES

The files from which it extracts exceptions should be synaxically correct, and
all modules it I<require>s or I<use>s should be present. If the modules the
files from which it extracts exceptions are in non-standard locations,
you should specify these locations using B<-p> switch - e.g. to extract 
exceptions from file, that uses modules located in directory F<lib>, add
the following to your commandline: S<C<-p 'perl -Ilib'>>.

It's impossible to create a list of exceptions by using only B<-d> switch
for modules referenced using C<require> operator. You will have to specify 
module name prefix to be able to extract symbols from such modules.

This script extracts just symbol names, without any type information - i.e.
if some module exports just two symbols - I<$foo> and I<%foo>, the resultant 
exceptions file will contain just one line with word C<foo>. The strings passed
directly via B<-u>, B<-x> options and indirectly via files specified with
B<-u>, B<-x> should also be symbol names, without package name part and 
without type-specific prefix (I<$>, I<%>, etc) - e.g. I<bar>, not 
I<$bar> or I<$blah::bar> (though the script will try to extract bare symbol 
name).

On unix-like OSes, in case all perl source files use modules from standard
locations (F</usr/lib>, F</usr/local/lib>, etc) it's very convenient to 
invoke the F<gen-ident-exceptions.pl> script with the relative paths to the
perl source files, adding B<-d /> to the commandline. This way you won't
have to type common module prefixes your source file uses. On Windows same
applies, but appending B<-d C:> to the commandline instead (provided
the perl is installed on drive F<C:>).


=cut
die('this is a stub of gen-ident-exceptions.pl. To evaluate a product, ask Stunnix support for a real trial via email');
